import edu.jhu.pha.skyservice.*;
import org.apache.axis.attachments.*;
import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import java.io.*;

/*
 A simple DIME client which will pull down a requested image from
 SDDS DR1 Image Cutout WebService
 http://skyservice.pha.jhu.edu/DR1/ImgCutout/ImgCutout.asmx

*/

public class ImgCutoutClient {

   /**
    * simple main loop which parses command line to call the webservice
    * if you pass no args defaults are provided
    * otherwise you must specify
    * FileName RA DEC SCALE WIDTH HEIGHT [OPTIONS]
    *
    */
    public static void main(String[] args) throws Exception 
    {
	
	// Default input parameters

	String fname = "DR1Default.jpg";
	double ra= 195.162;
	double dec= 2.5;
	double scale = 0.396127;
	int width = 512;
	int height = 512;
	String opt="";

	// Validate input
	int ind =0;
	if (args.length ==7) {

		fname = args[ind++];
		ra = Double.parseDouble(args[ind++]);
		dec = Double.parseDouble(args[ind++]);
		scale = Double.parseDouble(args[ind++]);
		width = Integer.parseInt(args[ind++]);
		height = Integer.parseInt(args[ind++]);
		opt = args[ind++];

	} else if (args.length>0) {
			System.err.println("Pass no args to use the defaults otherwise pass:");
			System.err.println("FileName RA DEC SCALE WIDTH HEIGHT OPTIONS");
			System.exit(2);
		}

	ImgCutoutClient cl = new ImgCutoutClient();

        cl.retrieveImage(fname,ra,dec,scale,width,height,opt);
    }

    /**
     * Retrieve and save the image to a given file
     */
    public void retrieveImage(String fname, double ra, double dec, 
                          double scale, int width, int height , 
                          String opt ) throws Exception
    {
	DataHandler dh = getImage(ra,dec,scale,width,height,opt); 

	FileOutputStream fs = new FileOutputStream(fname);
	dh.writeTo(fs);
	fs.close();

	// need to delete temp file which Axis creates
	File f = new File(dh.getName());
	f.delete();
    }
   
   /**
     * Call the webservice and get the attachement out of it.
     * This is returned as DataHandler which is part of Javax.Activation.
     */
    public DataHandler getImage(double ra, double dec, double scale, 
                                int width, int height, 
                                String opt) throws Exception 
    {

	ImgCutoutLocator loc = new ImgCutoutLocator();

	ImgCutoutSoap imgProvider = loc.getImgCutoutSoap();

	imgProvider.dimeJpeg(ra, dec,scale, width, height, opt);

	//Not so nice but we must cast to a stub to get the attachements
	Object[] obj = ((org.apache.axis.client.Stub) imgProvider).getAttachments();

	//Assuming one attachemnt only
	if(obj.length != 1 ) throw new Exception ("Expected a single attachemnt but got " 
                                                  + obj.length);
		
	//Each Attachment is actually an AttachmentPart - we only have 1
	AttachmentPart p = (AttachmentPart) obj[0];
	
	//We get the dataHandler
	DataHandler dh = p.getActivationDataHandler();
	
	return dh;
     }

 

}
